
Device = 18F452
Declare Create_Coff On
Declare Optimiser_Level = 3
Declare Watchdog Off
Declare Dead_Code_Remove = On

Declare Xtal 20

Declare Optimiser_Level = 3						' Maximum optimiser
Declare Dead_Code_Remove = On


Declare  Float_Display_Type = Fast				' Enable Faster floating point display
Declare Float_Rounding = OFF					' Turn off floating point rounding for better speed

Declare Adin_Res	10					        ' Set the resolution to 10
Declare Adin_Tad    64_FOSC 			                ' Choose the RC osc for ADC SAMPLE_NUMBER
Declare Adin_Stime	50        			         ' Allow 100us for charge time

Declare Hserial_Baud = 9600         


Dim Q0             As Float
Dim Q1             As Float
Dim Q2             As Float

Dim FLOAT_TEMP     As Float
Dim real           As Float
Dim imag           As Float

Dim MAGNITUDE      As Word
Dim MAGNITUDE_0    As Word
Dim MAGNITUDE_1    As Word
Dim MAGNITUDE_2    As Word
Dim MAGNITUDE_3    As Word
Dim MAGNITUDE_4    As Word
Dim MAGNITUDE_5    As Word
Dim MAGNITUDE_6    As Word
Dim MAGNITUDE_7    As Word

Dim MAX_MAG_ROW    As Word
Dim MAX_MAG_COLUMN As Word
Dim ROW            As Byte
Dim COLUMN         As Byte

Dim DTMF_CODE_DETECTED As String * 1

Dim RAW_ADC_READING       As ADRESL.Word   
Dim TIMER1REG             As TMR1L.Word                         


Dim SAMPLE_NUMBER         As Byte
Dim SAMPLE_BANK_COMPLETE  As Bit
Dim PROC_SAMP  As Byte
Dim CHECK_FREQ As Byte


Symbol PEIE    INTCON.6                                 ' Peripheral Interrupt Enable
Symbol GIE     INTCON.7                                 ' Global Interrupt Enable

' declares TMR1
Symbol TMR1ON  T1CON.0                                  ' Timer1 ON
Symbol TMR1IF  PIR1.0                                   ' TMR1 interrupt flag
Symbol TMR1IE  PIE1.0                                   ' TMR1 interrupt enable

                                
Symbol _125uS_TICK = 64951                

Symbol _N = 105                                         ' number of samples to collect in a detection bank 

' Create the actual seriel buffer in high memory

Dim SAMPLES_BANK[_N] As Word 'At __START_OF_RESERVE_RAM	

Symbol PI = 3.14159265358979323                         ' PI constant
Symbol PIx2 = 6.28318530717958						    ' PI * 2

 Clear SAMPLE_NUMBER       
 Clear SAMPLE_BANK_COMPLETE   

'$define Fi_0     697      ' frequency 0
'$define Fi_1     770
'$define Fi_2     852
'$define Fi_3     941
'$define Fi_4     1209
'$define Fi_5     1336
'$define Fi_6     1477
'$define Fi_7     1633

'  PRECOMPUTED VALUES  for Sample rate = 8Khz, N=105
$define _coeff_0  1.716897587
$define _coeff_1  1.652477549
$define _coeff_2  1.582142069
$define _coeff_3  1.506142932
$define _coeff_4  1.151234131
$define _coeff_5  0.947737325
$define _coeff_6  0.840714457
$define _coeff_7  0.618033989
Dim COEFF          As Float        ' where x = 0 to 7

$define _sin_0    0.512899277
$define _sin_1    0.563320058
$define _sin_2    0.611724299
$define _sin_3    0.657938726
$define _sin_4    0.817719386
$define _sin_5    0.880595532
$define _sin_6    0.907358695
$define _sin_7    0.951056516
Dim SINE           As Float         ' where x = 0 to 7

$define _cos_0    0.858448794
$define _cos_1    0.826238774
$define _cos_2    0.791071035
$define _cos_3    0.753071466
$define _cos_4    0.575617066
$define _cos_5    0.473868662
$define _cos_6    0.420357228
$define _cos_7    0.309016994
Dim COSINE         As Float         ' where x = 0 to 7

On_Hardware_Interrupt GoTo Isr

Clear                     
    TRISA.0 = 1					   ' Setup bit-0 of PortA as an input
    
    T1CON   = %10000000            ' SET UP TMR1 TO HAVE 1:1 PRESCALER AND ACT AS A TIMER
    TMR1IF  = 0                    ' CLEAR TMR1 INTERRUPT FLAG
    TMR1IE  = 1                    ' ENABLE TMR1 AS PERIPHERAL INTERRUPT SOURCE
    TMR1ON  = 0                    ' TMR1 not running

    ADCON1 = %10000010			    ' Set 18f452 PORTA analog and right justify result

    TMR1ON = 0                                  ' TMR1 is off
    TIMER1REG = _125uS_TICK                     
    ENABLE_TMR1_INTERRUPT ()                   
    TMR1ON = 1                                  ' start TMR1

Do                                
 If SAMPLE_BANK_COMPLETE = 1 Then   
                
                 MAX_MAG_ROW = 2000
                 MAX_MAG_COLUMN = 2000
                 For CHECK_FREQ = 0 To 7         ' number of bins - 1
                     Select CHECK_FREQ
                            Case 0
                                 COEFF  = _coeff_0
                                 SINE   = _sin_0
                                 COSINE = _cos_0
                            Case 1
                                 COEFF = _coeff_1
                                 SINE   = _sin_1
                                 COSINE = _cos_1     
                            Case 2
                                 COEFF = _coeff_2
                                 SINE   = _sin_2
                                 COSINE = _cos_2     
                            Case 3
                                 COEFF = _coeff_3
                                 SINE   = _sin_3
                                 COSINE = _cos_3     
                            Case 4
                                 COEFF = _coeff_4
                                 SINE   = _sin_4
                                 COSINE = _cos_4
                            Case 5
                                 COEFF = _coeff_5
                                 SINE   = _sin_5
                                 COSINE = _cos_5
                            Case 6
                                 COEFF = _coeff_6
                                 SINE   = _sin_6
                                 COSINE = _cos_6
                            Case 7
                                 COEFF = _coeff_7
                                 SINE   = _sin_7
                                 COSINE = _cos_7     
                     EndSelect         
                     ' process Q0, Q1, Q2
                     Q1 = 0                                ' start with these two cleared
                     Q2 = 0
                     PROC_SAMP = 0                       
                     
                     Repeat
                         Q0 = COEFF  * Q1  
                         FLOAT_TEMP = Q2 + SAMPLES_BANK[PROC_SAMP]
                         Q0 = Q0 - FLOAT_TEMP
                         Q2 = Q1
                         Q1 = Q0
                         Inc  PROC_SAMP
                     Until PROC_SAMP = _N
         
                     FLOAT_TEMP = Q2 * COSINE
                     real = Q1 - FLOAT_TEMP
                     imag = Q2 * SINE
                     real = real * real
                     imag = imag * imag
                     FLOAT_TEMP = real + imag
                     MAGNITUDE = Sqr  FLOAT_TEMP
                   
                     Select CHECK_FREQ
                            Case 0
                                 MAGNITUDE_0 = MAGNITUDE
                                 If MAGNITUDE_0 > MAX_MAG_ROW Then
                                    MAX_MAG_ROW = MAGNITUDE
                                    ROW = 0
                                 EndIf 
                            Case 1
                                 MAGNITUDE_1 = MAGNITUDE
                                 If MAGNITUDE_1 > MAX_MAG_ROW Then
                                    MAX_MAG_ROW = MAGNITUDE
                                    ROW = 1
                                 EndIf
                            Case 2
                                 MAGNITUDE_2 = MAGNITUDE
                                 If MAGNITUDE_2 > MAX_MAG_ROW Then
                                    MAX_MAG_ROW = MAGNITUDE
                                    ROW = 2
                                 EndIf
                            Case 3
                                 MAGNITUDE_3 = MAGNITUDE
                                 If MAGNITUDE_3 > MAX_MAG_ROW Then
                                    MAX_MAG_ROW = MAGNITUDE
                                    ROW = 3
                                 EndIf
                            Case 4
                                 MAGNITUDE_4 = MAGNITUDE
                                 If MAGNITUDE_4 > MAX_MAG_COLUMN Then
                                    MAX_MAG_COLUMN = MAGNITUDE
                                    COLUMN = 4
                                 EndIf
                            Case 5
                                 MAGNITUDE_5 = MAGNITUDE
                                 If MAGNITUDE_5 > MAX_MAG_COLUMN Then
                                    MAX_MAG_COLUMN = MAGNITUDE
                                    COLUMN = 5
                                 EndIf
                            Case 6
                                 MAGNITUDE_6 = MAGNITUDE
                                 If MAGNITUDE_6 > MAX_MAG_COLUMN Then
                                    MAX_MAG_COLUMN = MAGNITUDE
                                    COLUMN = 6
                                 EndIf
                            Case 7
                                 MAGNITUDE_7 = MAGNITUDE
                                 If MAGNITUDE_7 > MAX_MAG_COLUMN Then
                                    MAX_MAG_COLUMN = MAGNITUDE
                                    COLUMN = 7
                                 EndIf
                     EndSelect
                 Next
                
                 DECODE_DTMF_CHAR()    
              
                 HRSOutLn "DTMF CHAR = ", DTMF_CODE_DETECTED
               
                 SAMPLE_BANK_COMPLETE = 0
                 TIMER1REG = _125uS_TICK  
                 TMR1IE   = 1
 EndIf
Loop   

Proc DECODE_DTMF_CHAR()
    If ROW = 0 Then
            Select COLUMN
                Case 4 
                 DTMF_CODE_DETECTED = "1" 
                Case 5
                 DTMF_CODE_DETECTED = "2" 
                Case 6
                 DTMF_CODE_DETECTED = "3" 
                Case 7
                 DTMF_CODE_DETECTED = "A" 
            EndSelect
       ElseIf ROW = 1 Then
            Select COLUMN
                Case 4
                 DTMF_CODE_DETECTED = "4" 
                Case 5
                 DTMF_CODE_DETECTED = "5" 
                Case 6
                 DTMF_CODE_DETECTED = "6" 
                Case 7
                 DTMF_CODE_DETECTED = "B" 
            EndSelect    
       ElseIf ROW = 2 Then
            Select COLUMN
                Case 4
                 DTMF_CODE_DETECTED = "7" 
                Case 5
                 DTMF_CODE_DETECTED = "8"  
                Case 6
                 DTMF_CODE_DETECTED = "9" 
                Case 7
                 DTMF_CODE_DETECTED = "C" 
            EndSelect
       ElseIf ROW = 3 Then
            Select COLUMN
                Case 4
                 DTMF_CODE_DETECTED = "*" 
                Case 5
                 DTMF_CODE_DETECTED = "0" 
                Case 6
                 DTMF_CODE_DETECTED = "#" 
                Case 7
                 DTMF_CODE_DETECTED = "D" 
            EndSelect
    EndIf
EndProc


Isr:
Context Save
If TMR1IF = 1 Then                                
                    TIMER1REG = _125uS_TICK                        ' reload timer1
        
                    SAMPLES_BANK[SAMPLE_NUMBER] = ADIn 0       ' read the result
           
                    Inc SAMPLE_NUMBER
                    
                    If SAMPLE_NUMBER = _N Then
                                               TMR1IE   = 0                                    
                                               SAMPLE_BANK_COMPLETE = 1           
                                               SAMPLE_NUMBER = 0
                    EndIf

                    Clear TMR1IF                                   ' clear timer1 int flag so we wait for new interrup
EndIf
  

Context Restore   

'------------------------------------------------------------------------------
Proc ENABLE_TMR1_INTERRUPT()

    INTCON = 0
    INTCON.7 = 1                                       ' Enable General Interrupts 
    INTCON.6 = 1                                       ' Enable Peripherals Interrupts 
    TMR1IF   = 0                                       ' Clear TMR1 Interrupt Flag
    TMR1IE   = 1                                       ' TMR1 Interrupt Enable
EndProc


Config_Start
  OSC = HS	    'HS oscillator
  OSCS = OFF	'Oscillator system clock switch option is disabled (main oscillator is source)
  PWRT = OFF	'PWRT disabled
  BOR = OFF	    'Brown-out Reset disabled
  BORV = 20  	'VBOR set to 2.0V
  WDT = OFF	    'WDT disabled (control is placed on the SWDTEN bit)
  WDTPS = 128	'1:128
  CCP2MUX = On	'CCP2 input/output is multiplexed with RC1
  STVR = OFF	'Stack Full/Underflow will not cause RESET
  LVP = OFF	    'Low Voltage ICSP disabled
  Debug = OFF	'Background Debugger disabled. RB6 and RB7 configured as general purpose I/O pins.
  Cp0 = OFF	    'Block 0 (000200-001FFFh) not code protected
  CP1 = OFF 	'Block 1 (002000-003FFFh) not code protected
  CP2 = OFF 	'Block 2 (004000-005FFFh) not code protected
  CP3 = OFF 	'Block 3 (006000-007FFFh) not code protected
  CPB = OFF 	'Boot Block (000000-0001FFh) not code protected
  CPD = OFF 	'Data EEPROM not code protected
  WRT0 = OFF	'Block 0 (000200-001FFFh) not write protected
  WRT1 = OFF	'Block 1 (002000-003FFFh) not write protected
  WRT2 = OFF	'Block 2 (004000-005FFFh) not write protected
  WRT3 = OFF	'Block 3 (006000-007FFFh) not write protected
  WRTC = OFF	'Configuration registers (300000-3000FFh) not write protected
  WRTB = OFF	'Boot Block (000000-0001FFh) not write protected
  WRTD = OFF	'Data EEPROM not write protected
  EBTR0 = OFF	'Block 0 (000200-001FFFh) not protected from Table Reads executed in other blocks
  EBTR1 = OFF	'Block 1 (002000-003FFFh) not protected from Table Reads executed in other blocks
  EBTR2 = OFF	'Block 2 (004000-005FFFh) not protected from Table Reads executed in other blocks
  EBTR3 = OFF	'Block 3 (006000-007FFFh) not protected from Table Reads executed in other blocks
  EBTRB = OFF	'Boot Block (000000-0001FFh) not protected from Table Reads executed in other blocks
Config_End


    
